from django import template

register = template.Library()


@register.inclusion_tag("portal/partials/game_banner.html", takes_context=True)
def game_banner(context, game_banner_name):
    """
    Registers the inclusion tag for the game banner partial.
    Takes in the name of the game banner.
    The template currently expects the following context elements:
    - title: the heading of the banner (usually the name of the game)
    - description: a short explanation of the game
    - button_text: the text shown on the button
    - button_link: the link that the button redirects to
    - ages: the text showing the ages the game caters to
    - background_image_class: the CSS class of the image to be shown in the banner's
    background
    """
    return context[game_banner_name]



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from django import template
from django.template.defaultfilters import floatformat

register = template.Library()


@register.filter(name="tableformat")
def tableformat(entry):

    if entry is None:
        return "-"
    elif is_numerical(entry):
        return floatformat(entry, -2)
    else:
        return entry


def is_numerical(str):
    try:
        float(str)
        return True
    except (ValueError, TypeError):
        return False


@register.inclusion_tag("portal/partials/resource_sheets_table.html")
def resource_sheets_table(table):
    """
    This function takes in a dictionary which mirrors the table of resource sheets for a key stage section.
    The dictionary has a starting session index, and the actual table content. The table content is a 2D list:
    a list of sessions, each holding a list of resource sheets.
    Some sessions have more resource sheets than others, as such the table has some rows which contain less columns.
    The lengthen_list function is called to add empty columns to the smaller rows.
    :param table: A dictionary containing a starting session index (integer) and the content of the table (2D list)
    :return: A dictionary containing the table with all rows now of the same length, and the starting session index.
    """
    table_content = table["content"]
    max_count = len(max(table_content, key=len))
    return {
        "table": [lengthen_list(max_count, column) for column in table_content],
        "starting_session_index": table["starting_session_index"],
    }


def lengthen_list(length, list):
    return list + [[]] * (length - len(list))



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2018, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
import json

from common.models import School, Teacher, Class
from django.contrib import messages as messages
from django.contrib.auth.decorators import login_required, user_passes_test
from django.core.urlresolvers import reverse_lazy
from django.db.models import Q
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from rest_framework.authentication import SessionAuthentication
from rest_framework.views import APIView

import common.permissions as permissions
from common import email_messages
from portal.forms.organisation import OrganisationJoinForm, OrganisationForm
from common.helpers.emails import send_email, NOTIFICATION_EMAIL


class OrganisationFuzzyLookup(APIView):
    authentication_classes = (SessionAuthentication,)
    permission_classes = (permissions.LoggedInAsTeacher,)

    def get(self, request):
        fuzzy_name = request.GET.get("fuzzy_name", None)
        school_data = []

        # The idea here is to return all schools that satisfy that each
        # part of the fuzzy_name (separated by spaces) occurs in either
        # school.name or school.postcode.
        # So it is an intersection of unions.

        if fuzzy_name and len(fuzzy_name) > 2:
            schools = School.objects.all()
            for part in fuzzy_name.split():
                schools = schools.filter(
                    Q(name__icontains=part) | Q(postcode__icontains=part)
                )

            self._search_schools(schools, school_data)

        return HttpResponse(json.dumps(school_data), content_type="application/json")

    def _search_schools(self, schools, school_data):
        for school in schools:
            admins = Teacher.objects.filter(school=school, is_admin=True)
            admin = admins.first()
            if admin:
                email = admin.new_user.email
                admin_domain = "*********" + email[email.find("@") :]
                school_data.append(
                    {
                        "id": school.id,
                        "name": school.name,
                        "postcode": school.postcode,
                        "admin_domain": admin_domain,
                    }
                )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(
    permissions.logged_in_as_teacher, login_url=reverse_lazy("teacher_login")
)
def organisation_create(request):

    teacher = request.user.new_teacher

    create_form = OrganisationForm(user=request.user)
    join_form = OrganisationJoinForm()

    if request.method == "POST":
        if "create_organisation" in request.POST:
            create_form = OrganisationForm(request.POST, user=request.user)
            if create_form.is_valid():
                data = create_form.cleaned_data
                name = data.get("name", "")
                postcode = data.get("postcode", "").upper()
                country = data.get("country", "")

                error, town, lat, lng = (
                    "",
                    "0",
                    "0",
                    "0",
                )  # lookup_coord(postcode, country)

                school = School.objects.create(
                    name=name,
                    postcode=postcode,
                    town=town,
                    latitude=lat,
                    longitude=lng,
                    country=country,
                )

                teacher.school = school
                teacher.is_admin = True
                teacher.save()

                messages.success(
                    request,
                    "The school or club '"
                    + teacher.school.name
                    + "' has been successfully added.",
                )

                return HttpResponseRedirect(reverse_lazy("onboarding-classes"))

        elif "join_organisation" in request.POST:
            process_join_form(
                request, teacher, OrganisationJoinForm, OrganisationJoinForm
            )

        else:
            return process_revoke_request(request, teacher)

    res = render(
        request,
        "portal/teach/onboarding_school.html",
        {"create_form": create_form, "join_form": join_form, "teacher": teacher},
    )

    return res


def compute_input_join_form(
    OrganisationJoinFormWithCaptcha, OrganisationJoinForm, using_captcha
):
    InputOrganisationJoinForm = (
        OrganisationJoinFormWithCaptcha if using_captcha else OrganisationJoinForm
    )
    return InputOrganisationJoinForm


def compute_output_join_form(
    OrganisationJoinFormWithCaptcha, OrganisationJoinForm, should_use_captcha
):
    OutputOrganisationJoinForm = (
        OrganisationJoinFormWithCaptcha if should_use_captcha else OrganisationJoinForm
    )
    return OutputOrganisationJoinForm


def send_pending_requests_emails(school, email_message):
    for admin in Teacher.objects.filter(school=school, is_admin=True):
        send_email(
            NOTIFICATION_EMAIL,
            [admin.new_user.email],
            email_message["subject"],
            email_message["message"],
        )


def process_join_form(
    request, teacher, InputOrganisationJoinForm, OutputOrganisationJoinForm
):
    join_form = InputOrganisationJoinForm(request.POST)
    if join_form.is_valid():
        school = get_object_or_404(School, id=join_form.cleaned_data["chosen_org"])

        teacher.pending_join_request = school
        teacher.save()

        email_message = email_messages.joinRequestPendingEmail(
            request, teacher.new_user.email
        )

        send_pending_requests_emails(school, email_message)

        email_message = email_messages.joinRequestSentEmail(request, school.name)
        send_email(
            NOTIFICATION_EMAIL,
            [teacher.new_user.email],
            email_message["subject"],
            email_message["message"],
        )

        messages.success(
            request,
            "Your request to join the school or club has been sent successfully.",
        )

        return render(
            request,
            "portal/teach/onboarding_school.html",
            {"school": school, "teacher": teacher},
        )

    else:
        join_form = OutputOrganisationJoinForm(request.POST)


def process_revoke_request(request, teacher):
    if "revoke_join_request" in request.POST:
        teacher.pending_join_request = None
        teacher.save()

        messages.success(
            request,
            "Your request to join the school or club has been revoked successfully.",
        )

        return HttpResponseRedirect(reverse_lazy("onboarding-organisation"))


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(
    permissions.logged_in_as_teacher, login_url=reverse_lazy("teacher_login")
)
def organisation_manage(request):
    return organisation_create(request)


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(
    permissions.logged_in_as_teacher, login_url=reverse_lazy("teacher_login")
)
def organisation_leave(request):
    teacher = request.user.new_teacher

    check_teacher_is_not_admin(teacher)

    if request.method == "POST":
        classes = Class.objects.filter(teacher=teacher)
        for klass in classes:
            teacher_id = request.POST.get(klass.access_code, None)
            if teacher_id:
                new_teacher = get_object_or_404(Teacher, id=teacher_id)
                klass.teacher = new_teacher
                klass.save()

        classes = Class.objects.filter(teacher=teacher)
        teachers = Teacher.objects.filter(school=teacher.school).exclude(id=teacher.id)

        if classes.exists():
            messages.info(
                request,
                "You still have classes, you must first move them to another teacher within your school or club.",
            )
            return render(
                request,
                "portal/teach/teacher_move_all_classes.html",
                {
                    "original_teacher": teacher,
                    "classes": classes,
                    "teachers": teachers,
                    "submit_button_text": "Move classes and leave",
                },
            )

        teacher.school = None
        teacher.save()

        messages.success(request, "You have successfully left the school or club.")

        return HttpResponseRedirect(reverse_lazy("onboarding-organisation"))


def check_teacher_is_not_admin(teacher):
    if teacher.is_admin:
        raise Http404



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from builtins import object

from common.models import School
from django import forms
from django.core.exceptions import ObjectDoesNotExist
from django.core.validators import EmailValidator
from django_countries.widgets import CountrySelectWidget


class OrganisationForm(forms.ModelForm):
    class Meta(object):

        model = School
        fields = ["name", "postcode", "country"]
        labels = {
            "name": "Name of your school or club",
            "postcode": "Postcode",
            "country": "Country",
        }
        widgets = {
            "name": forms.TextInput(
                attrs={
                    "autocomplete": "off",
                    "placeholder": "Name of your school or club",
                }
            ),
            "postcode": forms.TextInput(
                attrs={"autocomplete": "off", "placeholder": "Postcode"}
            ),
            "country": CountrySelectWidget(attrs={"class": "wide"}),
        }

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop("user", None)
        self.current_school = kwargs.pop("current_school", None)
        super(OrganisationForm, self).__init__(*args, **kwargs)
        self.fields["postcode"].strip = False

    def clean(self):
        name = self.cleaned_data.get("name", None)
        postcode = self.cleaned_data.get("postcode", None)

        if name and postcode:
            try:
                school = School.objects.get(name=name, postcode=postcode)
            except ObjectDoesNotExist:
                return self.cleaned_data

            if not self.current_school or self.current_school.id != school.id:
                raise forms.ValidationError(
                    "There is already a school or club registered with that name and postcode"
                )

        return self.cleaned_data

    def clean_name(self):
        name = self.cleaned_data.get("name", None)
        validator = EmailValidator()

        if name:
            try:
                validator(name)
                is_email = True
            except forms.ValidationError:
                is_email = False

            if is_email:
                raise forms.ValidationError(
                    "Please make sure your organisation name is valid"
                )

        return name

    def clean_postcode(self):
        postcode = self.cleaned_data.get("postcode", None)

        if postcode:
            if (
                len(postcode.replace(" ", "")) > 10
                or len(postcode.replace(" ", "")) == 0
            ):
                raise forms.ValidationError("Please enter a valid postcode or ZIP code")

        return postcode


class OrganisationJoinForm(forms.Form):
    fuzzy_name = forms.CharField(
        label="Search for school or club by name or postcode",
        widget=forms.TextInput(attrs={"placeholder": "Enrico Fermi High School"}),
    )

    # Note: the reason this is a CharField rather than a ChoiceField is to avoid having to
    # provide choices which was problematic given that the options are dynamically generated.
    chosen_org = forms.CharField(
        label="Select school or club", widget=forms.Select(attrs={"class": "wide"})
    )

    def clean_chosen_org(self):
        chosen_org = self.cleaned_data.get("chosen_org", None)

        if chosen_org and not School.objects.filter(id=int(chosen_org)).exists():
            raise forms.ValidationError("That school or club was not recognised")

        return chosen_org



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import division

import json
from builtins import zip
from datetime import timedelta
from functools import partial, wraps

from aimmo.models import Game
from common.models import Class, Student, Teacher
from django.conf import settings
from django.contrib import messages as messages
from django.contrib.auth.decorators import login_required, user_passes_test
from django.contrib.staticfiles.storage import staticfiles_storage
from django.core.urlresolvers import reverse_lazy
from django.forms.formsets import formset_factory
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.utils import timezone
from django.views.decorators.http import require_POST
from past.utils import old_div
from reportlab.lib.colors import black
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import ParagraphStyle
from reportlab.lib.utils import ImageReader
from reportlab.pdfgen import canvas
from reportlab.platypus import Paragraph

from common import email_messages
from portal.forms.invite_teacher import InviteTeacherForm
from portal.forms.teach import (
    BaseTeacherDismissStudentsFormSet,
    BaseTeacherMoveStudentsDisambiguationFormSet,
    ClassCreationForm,
    ClassEditForm,
    ClassMoveForm,
    StudentCreationForm,
    TeacherDismissStudentsForm,
    TeacherEditStudentForm,
    TeacherMoveStudentDisambiguationForm,
    TeacherMoveStudentsDestinationForm,
    TeacherSetStudentPass,
)
from common.helpers.emails import INVITE_FROM, send_email, send_verification_email
from common.helpers.generators import (
    generate_access_code,
    generate_new_student_name,
    generate_password,
)
from common.permissions import logged_in_as_teacher
from portal.templatetags.app_tags import cloud_storage
from portal.views.teacher.pdfs import PDF_DATA


def get_links(pdf_name):
    links = PDF_DATA[pdf_name]["links"]
    link_titles = []
    for link in links:
        link = link.replace("_", " ")
        link_titles.append(link)

    return list(zip(links, link_titles))


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def materials_viewer(request, pdf_name):
    try:
        title = PDF_DATA[pdf_name]["title"]
        description = PDF_DATA[pdf_name]["description"]
        url = cloud_storage(PDF_DATA[pdf_name]["url"])
        page_origin = PDF_DATA[pdf_name]["page_origin"]

    except KeyError:
        raise Http404

    links = None
    video_link = None
    video_download_link = None

    if PDF_DATA[pdf_name]["links"] is not None:
        links = get_links(pdf_name)

    if "video" in PDF_DATA[pdf_name]:
        video_link = PDF_DATA[pdf_name]["video"]
        video_download_link = cloud_storage(PDF_DATA[pdf_name]["video_download_link"])

    return render(
        request,
        "portal/teach/viewer.html",
        {
            "title": title,
            "description": description,
            "url": url,
            "links": links,
            "video_link": video_link,
            "video_download_link": video_download_link,
            "page_origin": page_origin,
        },
    )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def default_solution(request, levelName):
    if 80 <= int(levelName) <= 91:
        return render(
            request, "portal/teach/teacher_solutionPY.html", {"levelName": levelName}
        )
    else:
        return render(
            request, "portal/teach/teacher_solution.html", {"levelName": levelName}
        )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_onboarding_create_class(request):
    """
    Onboarding view for creating a class (and organisation if there isn't one, yet)
    """
    teacher = request.user.new_teacher
    requests = Student.objects.filter(pending_class_request__teacher=teacher)

    if not teacher.school:
        return HttpResponseRedirect(reverse_lazy("onboarding-organisation"))

    if request.method == "POST":
        form = ClassCreationForm(request.POST)
        if form.is_valid():
            created_class = create_class(form, teacher)
            messages.success(
                request,
                "The class '{className}' has been created successfully.".format(
                    className=created_class.name
                ),
            )
            return HttpResponseRedirect(
                reverse_lazy(
                    "onboarding-class",
                    kwargs={"access_code": created_class.access_code},
                )
            )
    else:
        form = ClassCreationForm(initial={"classmate_progress": "False"})

    classes = Class.objects.filter(teacher=teacher)

    return render(
        request,
        "portal/teach/onboarding_classes.html",
        {"form": form, "classes": classes, "requests": requests},
    )


def create_class(form, teacher):
    classmate_progress = False
    if form.cleaned_data["classmate_progress"] == "True":
        classmate_progress = True
    klass = Class.objects.create(
        name=form.cleaned_data["class_name"],
        teacher=teacher,
        access_code=generate_access_code(),
        classmates_data_viewable=classmate_progress,
    )
    return klass


def process_edit_class(request, access_code, onboarding_done, next_url):
    """
    Handles student creation both during onboarding or on the class page
    """
    klass = get_object_or_404(Class, access_code=access_code)
    teacher = request.user.new_teacher
    students = Student.objects.filter(class_field=klass).order_by(
        "new_user__first_name"
    )

    check_user_is_authorised(request, klass)

    if request.method == "POST":
        new_students_form = StudentCreationForm(klass, request.POST)
        if new_students_form.is_valid():
            name_tokens = []
            for name in new_students_form.strippedNames:
                password = generate_password(8)
                name_tokens.append({"name": name, "password": password})

                new_student = Student.objects.schoolFactory(
                    klass=klass, name=name, password=password
                )

            return render(
                request,
                "portal/teach/onboarding_print.html",
                {
                    "class": klass,
                    "name_tokens": name_tokens,
                    "onboarding_done": onboarding_done,
                    "query_data": json.dumps(name_tokens),
                },
            )
    else:
        new_students_form = StudentCreationForm(klass)

    classes = Class.objects.filter(teacher=teacher)

    return render(
        request,
        next_url,
        {
            "class": klass,
            "classes": classes,
            "students": students,
            "new_students_form": new_students_form,
            "num_students": len(students),
        },
    )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_onboarding_edit_class(request, access_code):
    """
    Adding students to a class during the onboarding process
    """
    return process_edit_class(
        request,
        access_code,
        onboarding_done=False,
        next_url="portal/teach/onboarding_students.html",
    )


def check_user_is_authorised(request, klass):
    # check user authorised to see class
    if request.user.new_teacher != klass.teacher:
        raise Http404


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_view_class(request, access_code):
    """
    Adding students to a class after the onboarding process has been completed
    """
    return process_edit_class(
        request, access_code, onboarding_done=True, next_url="portal/teach/class.html"
    )


@require_POST
@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_delete_class(request, access_code):
    klass = get_object_or_404(Class, access_code=access_code)

    # check user authorised to see class
    if request.user.new_teacher != klass.teacher:
        raise Http404

    if Student.objects.filter(class_field=klass).exists():
        messages.info(
            request,
            "This class still has students, please remove or delete them all before deleting the class.",
        )
        return HttpResponseRedirect(
            reverse_lazy("view_class", kwargs={"access_code": access_code})
        )

    klass.delete()

    return HttpResponseRedirect(reverse_lazy("dashboard"))


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_delete_students(request, access_code):
    klass = get_object_or_404(Class, access_code=access_code)

    # check user is authorised to deal with class
    if request.user.new_teacher != klass.teacher:
        raise Http404

    # get student objects for students to be deleted, confirming they are in the class
    student_ids = json.loads(request.POST.get("transfer_students", "[]"))
    students = [
        get_object_or_404(Student, id=i, class_field=klass) for i in student_ids
    ]

    # Delete all of the students
    for student in students:
        student.new_user.delete()

    return HttpResponseRedirect(
        reverse_lazy("view_class", kwargs={"access_code": access_code})
    )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_edit_class(request, access_code):
    """
    Editing class details
    """
    klass = get_object_or_404(Class, access_code=access_code)

    # check user authorised to see class
    if request.user.new_teacher != klass.teacher:
        raise Http404

    external_requests_message = klass.get_requests_message()

    if request.method == "POST":
        form = ClassEditForm(request.POST)
        if form.is_valid():
            return process_edit_class_form(request, klass, form)

    else:
        form = ClassEditForm(
            initial={
                "name": klass.name,
                "classmate_progress": klass.classmates_data_viewable,
            }
        )

    return render(
        request,
        "portal/teach/teacher_edit_class.html",
        {
            "form": form,
            "class": klass,
            "external_requests_message": external_requests_message,
        },
    )


def process_edit_class_form(request, klass, form):
    name = form.cleaned_data["name"]
    classmate_progress = False

    if form.cleaned_data["classmate_progress"] == "True":
        classmate_progress = True
    external_requests_setting = form.cleaned_data["external_requests"]
    if external_requests_setting != "":
        # Change submitted for external requests
        hours = int(external_requests_setting)
        if hours == 0:
            # Setting to off
            klass.always_accept_requests = False
            klass.accept_requests_until = None
            messages.info(
                request,
                "Class set successfully to never receive requests from external students.",
            )

        elif hours < 1000:
            # Setting to number of hours
            klass.always_accept_requests = False
            klass.accept_requests_until = timezone.now() + timedelta(hours=hours)
            messages.info(
                request,
                "Class set successfully to receive requests from external students until "
                + klass.accept_requests_until.strftime("%d-%m-%Y %H:%M")
                + " "
                + timezone.get_current_timezone_name(),
            )

        else:
            # Setting to always on
            klass.always_accept_requests = True
            klass.accept_requests_until = None
            messages.info(
                request,
                "Class set successfully to always receive requests from external students (not recommended)",
            )

    klass.name = name
    klass.classmates_data_viewable = classmate_progress
    klass.save()

    messages.success(request, "The class's settings have been changed successfully.")

    return HttpResponseRedirect(
        reverse_lazy("view_class", kwargs={"access_code": klass.access_code})
    )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_edit_student(request, pk):
    """
    Changing a student's details
    """
    student = get_object_or_404(Student, id=pk)

    check_if_reset_authorised(request, student)

    name_form = TeacherEditStudentForm(
        student, initial={"name": student.new_user.first_name}
    )

    password_form = TeacherSetStudentPass()
    set_password_mode = False

    if request.method == "POST":
        if "update_details" in request.POST:
            name_form = TeacherEditStudentForm(student, request.POST)
            if name_form.is_valid():
                name = name_form.cleaned_data["name"]
                student.new_user.first_name = name
                student.new_user.save()
                student.save()

                messages.success(
                    request, "The student's details have been changed successfully."
                )

        else:
            password_form = TeacherSetStudentPass(request.POST)
            if password_form.is_valid():
                return process_reset_password_form(request, student, password_form)
            set_password_mode = True

    return render(
        request,
        "portal/teach/teacher_edit_student.html",
        {
            "name_form": name_form,
            "password_form": password_form,
            "student": student,
            "class": student.class_field,
            "set_password_mode": set_password_mode,
        },
    )


def process_reset_password_form(request, student, password_form):
    # check not default value for CharField
    new_password = password_form.cleaned_data["password"]
    if new_password:
        student.new_user.set_password(new_password)
        student.new_user.save()
        name_pass = [{"name": student.new_user.first_name, "password": new_password}]
        return render(
            request,
            "portal/teach/teacher_student_reset.html",
            {
                "student": student,
                "class": student.class_field,
                "password": new_password,
                "query_data": json.dumps(name_pass),
            },
        )


def check_if_reset_authorised(request, student):
    # check user is authorised to edit student
    if request.user.new_teacher != student.class_field.teacher:
        raise Http404


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_student_reset(request, pk):
    """
    Reset a student's password
    """
    student = get_object_or_404(Student, id=pk)

    # check user is authorised to edit student
    if request.user.new_teacher != student.class_field.teacher:
        raise Http404

    new_password = generate_password(8)
    student.new_user.set_password(new_password)
    student.new_user.save()
    name_pass = [{"name": student.new_user.first_name, "password": new_password}]

    return render(
        request,
        "portal/teach/teacher_student_reset.html",
        {
            "student": student,
            "class": student.class_field,
            "password": new_password,
            "query_data": json.dumps(name_pass),
        },
    )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_dismiss_students(request, access_code):
    """
    Dismiss a student (make them independent)
    """
    klass = get_object_or_404(Class, access_code=access_code)

    check_if_dismiss_authorised(request, klass)

    # get student objects for students to be dismissed, confirming they are in the class
    student_ids = json.loads(request.POST.get("transfer_students", "[]"))
    students = [
        get_object_or_404(Student, id=i, class_field=klass) for i in student_ids
    ]

    TeacherDismissStudentsFormSet = formset_factory(
        wraps(TeacherDismissStudentsForm)(partial(TeacherDismissStudentsForm)),
        extra=0,
        formset=BaseTeacherDismissStudentsFormSet,
    )

    if is_right_dismiss_form(request):
        formset = TeacherDismissStudentsFormSet(request.POST)
        if formset.is_valid():
            return process_dismiss_student_form(request, formset, klass, access_code)

    else:
        initial_data = [
            {
                "orig_name": student.new_user.first_name,
                "name": generate_new_student_name(student.new_user.first_name),
                "email": "",
            }
            for student in students
        ]

        formset = TeacherDismissStudentsFormSet(initial=initial_data)

    return render(
        request,
        "portal/teach/teacher_dismiss_students.html",
        {"formset": formset, "class": klass, "students": students},
    )


def check_if_dismiss_authorised(request, klass):
    # check user is authorised to deal with class
    if request.user.new_teacher != klass.teacher:
        raise Http404


def is_right_dismiss_form(request):
    return request.method == "POST" and "submit_dismiss" in request.POST


def process_dismiss_student_form(request, formset, klass, access_code):
    for data in formset.cleaned_data:
        student = get_object_or_404(
            Student, class_field=klass, new_user__first_name__iexact=data["orig_name"]
        )

        student.class_field = None
        student.new_user.first_name = data["name"]
        student.new_user.username = data["name"]
        student.new_user.email = data["email"]
        student.save()
        student.new_user.save()
        student.new_user.userprofile.save()

        send_verification_email(request, student.new_user)

    messages.success(
        request, "The students have been removed successfully from the class."
    )
    return HttpResponseRedirect(
        reverse_lazy("view_class", kwargs={"access_code": access_code})
    )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_class_password_reset(request, access_code):
    """
    Reset passwords for one or more students
    """
    klass = get_object_or_404(Class, access_code=access_code)

    # check user authorised to see class
    if request.user.new_teacher != klass.teacher:
        raise Http404

    student_ids = json.loads(request.POST.get("transfer_students", "[]"))
    students = [
        get_object_or_404(Student, id=i, class_field=klass) for i in student_ids
    ]

    name_tokens = []
    for student in students:
        password = generate_password(8)
        name_tokens.append({"name": student.new_user.first_name, "password": password})
        student.new_user.set_password(password)
        student.new_user.save()

    return render(
        request,
        "portal/teach/onboarding_print.html",
        {
            "class": klass,
            "onboarding_done": True,
            "passwords_reset": True,
            "name_tokens": name_tokens,
            "query_data": json.dumps(name_tokens),
        },
    )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_move_class(request, access_code):
    """
    Move a class to another teacher
    """
    klass = get_object_or_404(Class, access_code=access_code)
    old_teacher = klass.teacher
    other_teachers = Teacher.objects.filter(school=old_teacher.school).exclude(
        user=old_teacher.user
    )

    # check user authorised to see class
    if request.user.new_teacher != old_teacher:
        raise Http404

    if request.method == "POST":
        form = ClassMoveForm(other_teachers, request.POST)
        if form.is_valid():
            new_teacher_id = form.cleaned_data["new_teacher"]
            new_teacher = get_object_or_404(Teacher, id=new_teacher_id)

            klass.teacher = new_teacher
            klass.save()

            messages.success(
                request,
                "The class has been successfully assigned to a different teacher.",
            )
            return HttpResponseRedirect(reverse_lazy("dashboard"))
    else:
        form = ClassMoveForm(other_teachers)
    return render(
        request, "portal/teach/teacher_move_class.html", {"form": form, "class": klass}
    )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_move_students(request, access_code):
    """
    Move students
    """
    klass = get_object_or_404(Class, access_code=access_code)

    # check user is authorised to deal with class
    if request.user.new_teacher != klass.teacher:
        raise Http404

    transfer_students = request.POST.get("transfer_students", "[]")

    school = klass.teacher.school

    # get classes in same school
    classes = school.classes()
    classes.remove(klass)

    form = TeacherMoveStudentsDestinationForm(classes)

    return render(
        request,
        "portal/teach/teacher_move_students.html",
        {"transfer_students": transfer_students, "old_class": klass, "form": form},
    )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_move_students_to_class(request, access_code):
    """
    Disambiguation for moving students (teacher gets to rename the students to avoid clashes)
    """
    old_class = get_object_or_404(Class, access_code=access_code)
    new_class_id = request.POST.get("new_class", None)
    new_class = get_object_or_404(Class, id=new_class_id)

    check_if_move_authorised(request, old_class, new_class)

    transfer_students_ids = json.loads(request.POST.get("transfer_students", "[]"))

    # get student objects for students to be transferred, confirming they are in the old class still
    transfer_students = [
        get_object_or_404(Student, id=i, class_field=old_class)
        for i in transfer_students_ids
    ]

    # get new class' students
    new_class_students = Student.objects.filter(class_field=new_class).order_by(
        "new_user__first_name"
    )

    TeacherMoveStudentDisambiguationFormSet = formset_factory(
        wraps(TeacherMoveStudentDisambiguationForm)(
            partial(TeacherMoveStudentDisambiguationForm)
        ),
        extra=0,
        formset=BaseTeacherMoveStudentsDisambiguationFormSet,
    )

    if is_right_move_form(request):
        formset = TeacherMoveStudentDisambiguationFormSet(new_class, request.POST)
        if formset.is_valid():
            return process_move_students_form(request, formset, old_class, new_class)
    else:
        # format the students for the form
        initial_data = [
            {
                "orig_name": student.new_user.first_name,
                "name": student.new_user.first_name,
            }
            for student in transfer_students
        ]

        formset = TeacherMoveStudentDisambiguationFormSet(
            new_class, initial=initial_data
        )

    return render(
        request,
        "portal/teach/teacher_move_students_to_class.html",
        {
            "formset": formset,
            "old_class": old_class,
            "new_class": new_class,
            "new_class_students": new_class_students,
            "transfer_students": transfer_students,
        },
    )


def check_if_move_authorised(request, old_class, new_class):
    # check user is authorised to deal with class
    if request.user.new_teacher != old_class.teacher:
        raise Http404

    # check teacher authorised to transfer to new class
    if request.user.new_teacher.school != new_class.teacher.school:
        raise Http404


def is_right_move_form(request):
    return request.method == "POST" and "submit_disambiguation" in request.POST


def process_move_students_form(request, formset, old_class, new_class):
    old_teacher = old_class.teacher
    new_teacher = new_class.teacher

    for name_update in formset.cleaned_data:
        student = get_object_or_404(
            Student,
            class_field=old_class,
            new_user__first_name__iexact=name_update["orig_name"],
        )
        student.class_field = new_class
        student.new_user.first_name = name_update["name"]

        student.save()
        student.new_user.save()

    messages.success(request, "The students have been transferred successfully.")
    return HttpResponseRedirect(
        reverse_lazy("view_class", kwargs={"access_code": old_class.access_code})
    )


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_print_reminder_cards(request, access_code):
    response = HttpResponse(content_type="application/pdf")
    response["Content-Disposition"] = 'filename="student_reminder_cards.pdf"'

    p = canvas.Canvas(response, pagesize=A4)

    # Define constants that determine the look of the cards
    PAGE_WIDTH, PAGE_HEIGHT = A4
    PAGE_MARGIN = old_div(PAGE_WIDTH, 32)
    INTER_CARD_MARGIN = old_div(PAGE_WIDTH, 64)
    CARD_PADDING = old_div(PAGE_WIDTH, 48)

    NUM_X = 2
    NUM_Y = 4

    CARD_WIDTH = old_div(
        (PAGE_WIDTH - PAGE_MARGIN * 2 - INTER_CARD_MARGIN * (NUM_X - 1)), NUM_X
    )
    CARD_HEIGHT = old_div(
        (PAGE_HEIGHT - PAGE_MARGIN * 2 - INTER_CARD_MARGIN * (NUM_Y - 1)), NUM_Y
    )

    HEADER_HEIGHT = CARD_HEIGHT * 0.16
    FOOTER_HEIGHT = CARD_HEIGHT * 0.1

    CARD_INNER_WIDTH = CARD_WIDTH - CARD_PADDING * 2
    CARD_INNER_HEIGHT = CARD_HEIGHT - CARD_PADDING * 2 - HEADER_HEIGHT - FOOTER_HEIGHT

    CARD_IMAGE_WIDTH = CARD_INNER_WIDTH * 0.25

    CORNER_RADIUS = old_div(CARD_WIDTH, 32)

    # Setup various character images to cycle round
    CHARACTER_FILES = [
        "portal/img/dee.png",
        "portal/img/kirsty.png",
        "portal/img/wes.png",
        "portal/img/nigel.png",
        "portal/img/phil.png",
    ]
    CHARACTERS = []

    logo_image = ImageReader(
        staticfiles_storage.path("portal/img/logo_c4l_reminder_card.png")
    )

    for character_file in CHARACTER_FILES:
        character_image = ImageReader(staticfiles_storage.path(character_file))
        character_height = CARD_INNER_HEIGHT
        character_width = CARD_IMAGE_WIDTH
        character_height = old_div(
            character_width * character_image.getSize()[1], character_image.getSize()[0]
        )
        if character_height > CARD_INNER_HEIGHT:
            character_height = CARD_INNER_HEIGHT
            character_width = old_div(
                character_height * character_image.getSize()[0],
                character_image.getSize()[1],
            )
        character = {
            "image": character_image,
            "height": character_height,
            "width": character_width,
        }
        CHARACTERS.append(character)

    klass = get_object_or_404(Class, access_code=access_code)
    # Check auth
    if klass.teacher.new_user != request.user:
        raise Http404

    COLUMN_WIDTH = (CARD_INNER_WIDTH - CARD_IMAGE_WIDTH) * 0.45

    # Work out the data we're going to display, use data from the query string
    # if given, else display everyone in the class without passwords
    student_data = []

    student_data = get_student_data(request, klass, student_data)

    # Now draw everything
    x = 0
    y = 0

    def drawParagraph(text, position):
        style = ParagraphStyle("test")
        style.font = "Helvetica-Bold"

        font_size = 16
        while font_size > 0:
            style.fontSize = font_size
            style.leading = font_size

            para = Paragraph(text, style)
            (para_width, para_height) = para.wrap(
                CARD_INNER_WIDTH - COLUMN_WIDTH - CARD_IMAGE_WIDTH, CARD_INNER_HEIGHT
            )

            if para_height <= 48:
                para.drawOn(
                    p,
                    inner_left + COLUMN_WIDTH,
                    inner_bottom
                    + CARD_INNER_HEIGHT * position
                    + 8
                    - old_div(para_height, 2),
                )
                return

            font_size = font_size - 1

    current_student_count = 0
    for student in student_data:
        character_index = current_student_count % len(CHARACTERS)

        left = PAGE_MARGIN + x * CARD_WIDTH + x * INTER_CARD_MARGIN
        bottom = (
            PAGE_HEIGHT - PAGE_MARGIN - (y + 1) * CARD_HEIGHT - y * INTER_CARD_MARGIN
        )

        inner_left = left + CARD_PADDING
        inner_bottom = bottom + CARD_PADDING + FOOTER_HEIGHT

        header_bottom = bottom + CARD_HEIGHT - HEADER_HEIGHT
        footer_bottom = bottom

        # header rect
        p.setFillColorRGB(0.0, 0.027, 0.172)
        p.setStrokeColorRGB(0.0, 0.027, 0.172)
        p.roundRect(
            left, header_bottom, CARD_WIDTH, HEADER_HEIGHT, CORNER_RADIUS, fill=1
        )
        p.rect(left, header_bottom, CARD_WIDTH, old_div(HEADER_HEIGHT, 2), fill=1)

        # footer rect
        p.roundRect(left, bottom, CARD_WIDTH, FOOTER_HEIGHT, CORNER_RADIUS, fill=1)
        p.rect(
            left,
            bottom + old_div(FOOTER_HEIGHT, 2),
            CARD_WIDTH,
            old_div(FOOTER_HEIGHT, 2),
            fill=1,
        )

        # outer box
        p.setStrokeColor(black)
        p.roundRect(left, bottom, CARD_WIDTH, CARD_HEIGHT, CORNER_RADIUS)

        # header image
        p.drawImage(
            logo_image,
            inner_left,
            header_bottom + 5,
            CARD_INNER_WIDTH,
            HEADER_HEIGHT * 0.6,
        )

        # footer text
        p.setFont("Helvetica", 10)
        p.drawCentredString(
            inner_left + old_div(CARD_INNER_WIDTH, 2),
            footer_bottom + FOOTER_HEIGHT * 0.32,
            settings.CODEFORLIFE_WEBSITE,
        )

        # left hand side writing
        p.setFillColor(black)
        p.setFont("Helvetica", 12)
        p.drawString(inner_left, inner_bottom + CARD_INNER_HEIGHT * 0.12, "Password:")
        p.drawString(inner_left, inner_bottom + CARD_INNER_HEIGHT * 0.45, "Class Code:")
        p.drawString(inner_left, inner_bottom + CARD_INNER_HEIGHT * 0.78, "Name:")

        # right hand side writing
        drawParagraph(student["password"], 0.10)
        drawParagraph(klass.access_code, 0.43)
        drawParagraph(student["name"], 0.76)

        # character image
        character = CHARACTERS[character_index]
        p.drawImage(
            character["image"],
            inner_left + CARD_INNER_WIDTH - character["width"],
            inner_bottom,
            character["width"],
            character["height"],
            mask="auto",
        )

        x = (x + 1) % NUM_X
        y = compute_show_page_character(p, x, y, NUM_Y)
        current_student_count += 1

    compute_show_page_end(p, x, y)

    p.save()
    return response


def get_student_data(request, klass, student_data):
    if request.method == "POST":
        student_data = json.loads(request.POST.get("data", "[]"))

    else:
        students = Student.objects.filter(class_field=klass)

        for student in students:
            student_data.append(
                {"name": student.new_user.first_name, "password": "__________"}
            )

    return student_data


def compute_show_page_character(p, x, y, NUM_Y):
    if x == 0:
        y = (y + 1) % NUM_Y
        if y == 0:
            p.showPage()
    return y


def compute_show_page_end(p, x, y):
    if x != 0 or y != 0:
        p.showPage()


def invite_teacher(request):
    if request.method == "POST":
        invite_teacher_form = InviteTeacherForm(data=request.POST)
        if invite_teacher_form.is_valid():
            email_address = invite_teacher_form.cleaned_data["email"]
            email_message = email_messages.inviteTeacherEmail(request)
            send_email(
                INVITE_FROM,
                [email_address],
                email_message["subject"],
                email_message["message"],
            )
            return render(request, "portal/email_invitation_sent.html")

    return render(
        request, "portal/teach/invite.html", {"invite_form": InviteTeacherForm()}
    )



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

import time

from django.core import mail
from selenium.webdriver.support.wait import WebDriverWait

from common.tests.utils import email as email_utils
from common.tests.utils.classes import create_class_directly
from common.tests.utils.student import (
    create_independent_student,
    create_independent_student_directly,
    create_school_student_directly,
    signup_duplicate_independent_student_fail,
    submit_independent_student_signup_form,
)
from common.tests.utils.teacher import signup_teacher_directly
from portal.tests.utils.organisation import create_organisation_directly

from .base_test import BaseTest
from .pageObjects.portal.home_page import HomePage
from .utils.messages import (
    is_email_verified_message_showing,
    is_indep_student_join_request_received_message_showing,
    is_indep_student_join_request_revoked_message_showing,
    is_student_details_updated_message_showing,
)


class TestIndependentStudent(BaseTest):
    def test_signup_without_newsletter(self):
        page = self.go_to_homepage()
        page, _, _, _, _ = create_independent_student(page)
        assert is_email_verified_message_showing(self.selenium)

    def test_signup_with_newsletter(self):
        page = self.go_to_homepage()
        page, _, _, _, _ = create_independent_student(page, newsletter=True)
        assert is_email_verified_message_showing(self.selenium)

    def test_signup_duplicate_email_failure(self):
        page = self.go_to_homepage()
        page, _, _, email, _ = create_independent_student(page)
        assert is_email_verified_message_showing(self.selenium)

        page = self.go_to_homepage()
        page, _, _, _, _ = signup_duplicate_independent_student_fail(
            page, duplicate_email=email
        )

        assert len(mail.outbox) == 1
        assert mail.outbox[0].subject == "Code for Life: Duplicate account error"

        assert page.__class__.__name__ == "IndependentStudentLoginPage"

    def test_signup_duplicate_username_failure(self):
        username, _, _ = create_independent_student_directly()

        page = self.go_to_homepage()
        page, _, _, _, _ = signup_duplicate_independent_student_fail(
            page, duplicate_username=username
        )

        assert len(mail.outbox) == 1
        assert mail.outbox[0].subject == "Code for Life: Username already taken"

        assert page.__class__.__name__ == "IndependentStudentLoginPage"

    def test_signup_failure_short_password(self):
        page = self.go_to_homepage()
        page = submit_independent_student_signup_form(page, password="test")
        assert page.has_independent_student_signup_failed(
            "Password not strong enough, consider using at least 8 characters, upper and lower case letters, and numbers"
        )

    def test_signup_failure_common_password(self):
        page = self.go_to_homepage()
        page = submit_independent_student_signup_form(page, password="Password1")
        assert page.has_independent_student_signup_failed(
            "Password not strong enough, consider using at least 8 characters, upper and lower case letters, and numbers"
        )

    def test_signup_invalid_name(self):
        page = self.go_to_homepage().go_to_signup_page()
        page = page.independent_student_signup(
            "Florian!", "Florian", "e@mail.com", "Password2", "Password2", success=False
        )

        assert self.is_signup_page(page)
        assert page.has_independent_student_signup_failed(
            "Names may only contain letters, numbers, dashes, underscores, and spaces."
        )

    def test_signup_invalid_username(self):
        page = self.go_to_homepage().go_to_signup_page()
        page = page.independent_student_signup(
            "Florian", "///", "e@mail.com", "Password2", "Password2", success=False
        )

        assert self.is_signup_page(page)
        assert page.has_independent_student_signup_failed(
            "Usernames may only contain letters, numbers, dashes, and underscores."
        )

    def test_signup_password_do_not_match(self):
        page = self.go_to_homepage().go_to_signup_page()
        page = page.independent_student_signup(
            "Florian", "Florian", "e@mail.com", "Password2", "Password3", success=False
        )

        assert self.is_signup_page(page)
        assert page.has_independent_student_signup_failed("Your passwords do not match")

    def test_login_failure(self):
        page = self.go_to_homepage()
        page = page.go_to_independent_student_login_page()
        page = page.independent_student_login_failure(
            "Non existent username", "Incorrect password"
        )

        assert page.has_login_failed(
            "independent_student_login_form",
            "Please enter a correct username and password. Note that both fields may be case-sensitive.",
        )

    def test_login_success(self):
        page = self.go_to_homepage()
        page, _, username, _, password = create_independent_student(page)
        page = page.independent_student_login(username, password)
        assert page.__class__.__name__ == "PlayDashboardPage"

    def test_reset_password(self):
        page = self.go_to_homepage()

        page, name, username, _, _ = create_independent_student(page)
        page = self.get_to_forgotten_password_page()

        page.reset_username_submit(username)

        self.wait_for_email()

        page = email_utils.follow_reset_email_link(self.selenium, mail.outbox[0])

        new_password = "AnotherPassword12"

        page.student_reset_password(new_password)

        self.selenium.get(self.live_server_url)
        page = (
            self.go_to_homepage()
            .go_to_independent_student_login_page()
            .independent_student_login(username, new_password)
        )

        assert page.__class__.__name__ == "PlayDashboardPage"

        page = page.go_to_account_page()
        assert page.check_account_details({"name": name})

    def test_reset_password_fail(self):
        page = self.get_to_forgotten_password_page()

        fake_username = "fake_username"
        page.reset_username_submit(fake_username)

        time.sleep(5)

        assert len(mail.outbox) == 0

    def test_update_name_success(self):
        homepage = self.go_to_homepage()

        play_page, name, student_username, _, password = create_independent_student(
            homepage
        )

        page = play_page.independent_student_login(
            student_username, password
        ).go_to_account_page()

        assert page.check_account_details({"name": name})

        page = page.update_name_success("New name", password)

        assert self.is_dashboard(page)
        assert is_student_details_updated_message_showing(self.selenium)

    def test_update_name_failure(self):
        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(
            homepage
        )

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_account_page()
            .update_name_failure("Name!", password)
        )

        assert self.is_account_page(page)
        assert page.was_form_invalid(
            "student_account_form",
            "Names may only contain letters, numbers, dashes, underscores, and spaces.",
        )

    def test_change_email(self):
        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(
            homepage
        )

        page = play_page.independent_student_login(
            student_username, password
        ).go_to_account_page()

        new_email = "another-email@codeforlife.com"
        page = page.change_email(new_email, password)

        assert page.__class__.__name__ == "EmailVerificationNeededPage"
        assert is_student_details_updated_message_showing(self.selenium)

    def test_join_class_nonexistent_class(self):
        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(
            homepage
        )

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club_failure("AA123")
        )

        assert self.is_join_class_page(page)
        assert page.has_join_request_failed(
            "Cannot find the school or club and/or class"
        )

    def test_join_class_not_accepting_requests(self):
        teacher_email, _ = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        _, _, access_code = create_class_directly(teacher_email)
        create_school_student_directly(access_code)

        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(
            homepage
        )

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club_failure(access_code)
        )

        assert self.is_join_class_page(page)
        assert page.has_join_request_failed(
            "Cannot find the school or club and/or class"
        )

    def test_join_class_revoked(self):
        teacher_email, _ = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        klass, _, access_code = create_class_directly(teacher_email)
        create_school_student_directly(access_code)
        klass.always_accept_requests = True
        klass.save()

        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(
            homepage
        )

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club(access_code)
        )

        assert is_indep_student_join_request_received_message_showing(self.selenium)

        page.revoke_join_request()

        assert is_indep_student_join_request_revoked_message_showing(self.selenium)

    def test_join_class_accepted(self):
        teacher_email, teacher_password = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        klass, _, access_code = create_class_directly(teacher_email)
        create_school_student_directly(access_code)
        klass.always_accept_requests = True
        klass.save()

        homepage = self.go_to_homepage()

        (
            play_page,
            student_name,
            student_username,
            _,
            password,
        ) = create_independent_student(homepage)

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club(access_code)
        )

        page.logout()

        page = self.go_to_homepage()

        page = (
            page.go_to_teacher_login_page()
            .login(teacher_email, teacher_password)
            .accept_independent_join_request()
            .save(student_name)
            .return_to_class()
        )

        assert page.student_exists(student_name)

    def test_join_class_denied(self):
        teacher_email, teacher_password = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        klass, _, access_code = create_class_directly(teacher_email)
        create_school_student_directly(access_code)
        klass.always_accept_requests = True
        klass.save()

        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(
            homepage
        )

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club(access_code)
        )

        page.logout()

        page = self.go_to_homepage()

        dashboard_page = (
            page.go_to_teacher_login_page()
            .login(teacher_email, teacher_password)
            .deny_independent_join_request()
        )

        assert dashboard_page.has_no_independent_join_requests()

    def test_cannot_see_aimmo(self):
        page = self.go_to_homepage()
        page, _, username, _, password = create_independent_student(page)
        page = page.independent_student_login(username, password)

        assert page.element_does_not_exist_by_link_text("Kurono")

    def get_to_forgotten_password_page(self):
        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_independent_student_login_page()
            .go_to_indep_forgotten_password_page()
        )
        return page

    def wait_for_email(self):
        WebDriverWait(self.selenium, 2).until(lambda driver: len(mail.outbox) == 1)

    def is_signup_page(self, page):
        return page.__class__.__name__ == "SignupPage"

    def is_dashboard(self, page):
        return page.__class__.__name__ == "PlayDashboardPage"

    def is_account_page(self, page):
        return page.__class__.__name__ == "PlayAccountPage"

    def is_join_class_page(self, page):
        return page.__class__.__name__ == "JoinSchoolOrClubPage"



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from captcha.fields import ReCaptchaField
from captcha.widgets import ReCaptchaV2Invisible
from common.models import Student, Teacher
from django import forms
from django.contrib.auth import forms as django_auth_forms
from django.contrib.auth import get_user_model
from django.contrib.auth.models import User
from django.contrib.auth.tokens import default_token_generator
from django.contrib.sites.shortcuts import get_current_site
from django.core.mail import EmailMultiAlternatives
from django.template import loader
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode

from portal.helpers.password import form_clean_password


class PasswordResetSetPasswordForm(django_auth_forms.SetPasswordForm):
    def __init__(self, user, *args, **kwargs):
        super(PasswordResetSetPasswordForm, self).__init__(user, *args, **kwargs)
        self.fields["new_password1"].label = "Enter your new password"
        self.fields["new_password1"].widget.attrs[
            "placeholder"
        ] = "Try at least 8 characters, upper and lower case letters, and numbers"
        self.fields["new_password2"].label = "Confirm your new password"
        self.fields["new_password2"].widget.attrs[
            "placeholder"
        ] = "Please repeat your new password"

    def clean_new_password1(self):
        return form_clean_password(self, forms, "new_password1")


class TeacherPasswordResetForm(forms.Form):
    email = forms.EmailField(
        label="Email address",
        max_length=254,
        widget=forms.EmailInput(attrs={"placeholder": "my.email@address.com"}),
    )

    captcha = ReCaptchaField(widget=ReCaptchaV2Invisible)

    def clean_email(self):
        email = self.cleaned_data.get("email", None)
        self.username = ""
        # Check such an email exists
        if User.objects.filter(email=email).exists():
            teacher = Teacher.objects.filter(new_user__email=email)
            # Check such an email is associated with a teacher
            if teacher.exists():
                self.username = teacher[0].new_user.username

        return email

    def send_mail(
        self,
        subject_template_name,
        email_template_name,
        context,
        from_email,
        to_email,
        html_email_template_name=None,
    ):
        """
        Sends a django.core.mail.EmailMultiAlternatives to `to_email`.
        """
        subject = loader.render_to_string(subject_template_name, context)
        # Email subject *must not* contain newlines
        subject = "".join(subject.splitlines())
        body = loader.render_to_string(email_template_name, context)

        email_message = EmailMultiAlternatives(subject, body, from_email, [to_email])
        if html_email_template_name is not None:
            html_email = loader.render_to_string(html_email_template_name, context)
            email_message.attach_alternative(html_email, "text/html")

        email_message.send()

    def save(
        self,
        domain_override=None,
        subject_template_name="registration/password_reset_subject.txt",
        email_template_name="portal/reset_password_email.html",
        use_https=False,
        token_generator=default_token_generator,
        from_email=None,
        request=None,
        html_email_template_name=None,
    ):
        """
        Generates a one-use only link for resetting password and sends to the
        user.
        """
        UserModel = get_user_model()
        if self.username:
            active_users = UserModel._default_manager.filter(
                username=self.username, is_active=True
            )
            for user in active_users:
                # Make sure that no email is sent to a user that actually has
                # a password marked as unusable
                if not user.has_usable_password():
                    continue
                if not domain_override:
                    current_site = get_current_site(request)
                    site_name = current_site.name
                    domain = current_site.domain
                else:
                    site_name = domain = domain_override
                context = {
                    "email": user.email,
                    "domain": domain,
                    "site_name": site_name,
                    "uid": urlsafe_base64_encode(force_bytes(user.pk)),
                    "user": user,
                    "token": token_generator.make_token(user),
                    "protocol": compute_protocol(use_https),
                }

                self.send_mail(
                    subject_template_name,
                    email_template_name,
                    context,
                    from_email,
                    user.email,
                    html_email_template_name=html_email_template_name,
                )


class StudentPasswordResetForm(forms.Form):
    username = forms.CharField(
        label="Username", widget=forms.TextInput(attrs={"placeholder": "rosie_f"})
    )

    captcha = ReCaptchaField(widget=ReCaptchaV2Invisible)

    def clean_username(self):
        username = self.cleaned_data.get("username", None)
        user_filter = User.objects.filter(username=username)
        # Check such a username exists and such a username is not in use by a student part of a class/school
        if (
            not user_filter.exists()
            or not Student.objects.filter(
                class_field=None, new_user__username=username
            ).exists()
        ):
            username = ""
        return username

    def send_mail(
        self,
        subject_template_name,
        email_template_name,
        context,
        from_email,
        to_email,
        html_email_template_name=None,
    ):
        """
        Sends a django.core.mail.EmailMultiAlternatives to `to_email`.
        """
        subject = loader.render_to_string(subject_template_name, context)
        # Email subject *must not* contain newlines
        subject = "".join(subject.splitlines())
        body = loader.render_to_string(email_template_name, context)

        email_message = EmailMultiAlternatives(subject, body, from_email, [to_email])
        if html_email_template_name is not None:
            html_email = loader.render_to_string(html_email_template_name, context)
            email_message.attach_alternative(html_email, "text/html")

        email_message.send()

    def save(
        self,
        domain_override=None,
        subject_template_name="registration/password_reset_subject.txt",
        email_template_name="portal/reset_password_email.html",
        use_https=False,
        token_generator=default_token_generator,
        from_email=None,
        request=None,
        html_email_template_name=None,
    ):
        """
        Generates a one-use only link for resetting password and sends to the
        user.
        """
        UserModel = get_user_model()
        username = self.cleaned_data["username"]
        if username:
            active_users = UserModel._default_manager.filter(
                username=username, is_active=True
            )
            for user in active_users:
                # Make sure that no email is sent to a user that actually has
                # a password marked as unusable
                if not user.has_usable_password():
                    continue
                if not domain_override:
                    current_site = get_current_site(request)
                    site_name = current_site.name
                    domain = current_site.domain
                else:
                    site_name = domain = domain_override
                context = {
                    "email": user.email,
                    "domain": domain,
                    "site_name": site_name,
                    "uid": urlsafe_base64_encode(force_bytes(user.pk)),
                    "user": user,
                    "token": token_generator.make_token(user),
                    "protocol": compute_protocol(use_https),
                }

                self.send_mail(
                    subject_template_name,
                    email_template_name,
                    context,
                    from_email,
                    user.email,
                    html_email_template_name=html_email_template_name,
                )


def compute_protocol(use_https):
    return "https" if use_https else "http"



# -*- coding: utf-8 -*-
from django.shortcuts import render

from portal.strings.play_aimmo import (
    AIMMO_BENEFITS,
    AIMMO_MAIN_HEADLINE,
    AIMMO_PLAY_ONLINE_HEADLINE,
    AIMMO_RESOURCES_HEADLINE,
)


def play_aimmo(request):
    return render(
        request,
        "portal/play_aimmo.html",
        {
            "BENEFITS": AIMMO_BENEFITS,
            "HEADLINE": AIMMO_MAIN_HEADLINE,
            "TEACHING_RESOURCES": AIMMO_RESOURCES_HEADLINE,
            "PLAY_ONLINE": AIMMO_PLAY_ONLINE_HEADLINE,
        },
    )



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
import re
from builtins import map
from builtins import range
from builtins import str

from captcha.fields import ReCaptchaField
from captcha.widgets import ReCaptchaV2Invisible
from common.models import Student, stripStudentName
from django import forms
from django.contrib.auth import authenticate
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth.models import User

from portal.helpers.password import form_clean_password

choices = [
    ("Miss", "Miss"),
    ("Mrs", "Mrs"),
    ("Ms", "Ms"),
    ("Mr", "Mr"),
    ("Dr", "Dr"),
    ("Rev", "Rev"),
    ("Sir", "Sir"),
    ("Dame", "Dame"),
]


class TeacherSignupForm(forms.Form):

    teacher_title = forms.ChoiceField(
        label="Title", choices=choices, widget=forms.Select(attrs={"class": "wide"})
    )
    teacher_first_name = forms.CharField(
        label="First name",
        max_length=100,
        widget=forms.TextInput(attrs={"placeholder": "Grace"}),
    )
    teacher_last_name = forms.CharField(
        label="Last name",
        max_length=100,
        widget=forms.TextInput(attrs={"placeholder": "Hopper"}),
    )
    teacher_email = forms.EmailField(
        label="Email address",
        widget=forms.EmailInput(attrs={"placeholder": "grace.hopper@navy.mil"}),
    )

    newsletter_ticked = forms.BooleanField(
        widget=forms.CheckboxInput(), initial=False, required=False
    )

    teacher_password = forms.CharField(label="Password", widget=forms.PasswordInput())
    teacher_confirm_password = forms.CharField(
        label="Confirm Password", widget=forms.PasswordInput()
    )

    captcha = ReCaptchaField(widget=ReCaptchaV2Invisible)

    def clean_teacher_password(self):
        return form_clean_password(self, forms, "teacher_password")

    def clean(self):
        if any(self.errors):
            return

        password = self.cleaned_data.get("teacher_password", None)
        confirm_password = self.cleaned_data.get("teacher_confirm_password", None)

        check_passwords(password, confirm_password)

        return self.cleaned_data


class TeacherEditAccountForm(forms.Form):

    title = forms.ChoiceField(
        label="Title",
        choices=choices,
        widget=forms.Select(attrs={"placeholder": "Title", "class": "wide"}),
    )
    first_name = forms.CharField(
        label="First name",
        max_length=100,
        widget=forms.TextInput(attrs={"placeholder": "First name", "class": "fName"}),
    )
    last_name = forms.CharField(
        label="Last name",
        max_length=100,
        widget=forms.TextInput(attrs={"placeholder": "Last name", "class": "lName"}),
    )
    email = forms.EmailField(
        label="Change email address (optional)",
        required=False,
        widget=forms.EmailInput(attrs={"placeholder": "new.email@address.com"}),
    )
    password = forms.CharField(
        label="New password (optional)", required=False, widget=forms.PasswordInput
    )
    confirm_password = forms.CharField(
        label="Confirm new password", required=False, widget=forms.PasswordInput
    )
    current_password = forms.CharField(
        label="Current password", widget=forms.PasswordInput
    )

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super(TeacherEditAccountForm, self).__init__(*args, **kwargs)

    def clean_password(self):
        return form_clean_password(self, forms, "password")

    def clean(self):
        if any(self.errors):
            return

        password = self.cleaned_data.get("password", None)
        confirm_password = self.cleaned_data.get("confirm_password", None)
        current_password = self.cleaned_data.get("current_password", None)

        self.check_password_errors(password, confirm_password, current_password)

        return self.cleaned_data

    def check_password_errors(self, password, confirm_password, current_password):
        check_passwords(password, confirm_password)

        if not self.user.check_password(current_password):
            raise forms.ValidationError("Your current password was incorrect")


class TeacherLoginForm(AuthenticationForm):
    username = forms.EmailField(
        label="Email address",
        widget=forms.EmailInput(attrs={"placeholder": "my.email@address.com"}),
    )
    password = forms.CharField(label="Password", widget=forms.PasswordInput)

    captcha = ReCaptchaField(widget=ReCaptchaV2Invisible)

    def clean(self):
        email = self.cleaned_data.get("username", None)
        password = self.cleaned_data.get("password", None)

        if email and password:

            # Check it's a teacher and not a student using the same email address
            user = None

            user = self.find_user(email, user)

            user = authenticate(username=user.username, password=password)

            self.check_email_errors(user)

            self.user_cache = user

        return self.cleaned_data

    def find_user(self, email, user):
        users = User.objects.filter(email=email)

        for result in users:
            if hasattr(result, "userprofile") and hasattr(
                result.userprofile, "teacher"
            ):
                user = result
                break

        if user is None:
            raise forms.ValidationError("Incorrect email address or password")

        return user

    def check_email_errors(self, user):
        if user is None:
            raise forms.ValidationError("Incorrect email address or password")

        if not user.is_active:
            raise forms.ValidationError("User account has been deactivated")


class ClassCreationForm(forms.Form):
    classmate_choices = [("True", "Yes"), ("False", "No")]
    class_name = forms.CharField(
        label="Class Name", widget=forms.TextInput(attrs={"placeholder": "Lower KS2"})
    )
    classmate_progress = forms.ChoiceField(
        label="Allow students to see their classmates' progress?",
        choices=classmate_choices,
        widget=forms.Select(attrs={"class": "wide"}),
    )


class ClassEditForm(forms.Form):
    classmate_choices = [("True", "Yes"), ("False", "No")]
    # select dropdown choices for potentially limiting time in which external students may join
    # class
    # 0 value = don't allow
    # n value = allow for next n hours, n < 1000 hours
    # o/w = allow forever
    join_choices = [
        ("", "Don't change my current setting"),
        ("0", "Don't allow external requests to this class"),
        ("1", "Allow external requests to this class for the next hour"),
    ]
    join_choices.extend(
        [
            (
                str(hours),
                "Allow external requests to this class for the next "
                + str(hours)
                + " hours",
            )
            for hours in range(4, 24, 4)
        ]
    )
    join_choices.extend(
        [
            (
                str(days * 24),
                "Allow external requests to this class for the next "
                + str(days)
                + " days",
            )
            for days in range(2, 5)
        ]
    )
    join_choices.append(
        ("1000", "Always allow external requests to this class (not recommended)")
    )
    name = forms.CharField(
        label="Class Name", widget=forms.TextInput(attrs={"placeholder": "Class Name"})
    )
    classmate_progress = forms.ChoiceField(
        label="Allow students to see their classmates' progress?",
        choices=classmate_choices,
        widget=forms.Select(attrs={"class": "wide"}),
    )
    external_requests = forms.ChoiceField(
        label="Set up external requests to this class",
        required=False,
        choices=join_choices,
        widget=forms.Select(attrs={"class": "wide"}),
    )


class ClassMoveForm(forms.Form):
    new_teacher = forms.ChoiceField(
        label="Teachers", widget=forms.Select(attrs={"class": "wide"})
    )

    def __init__(self, teachers, *args, **kwargs):
        self.teachers = teachers
        teacher_choices = []
        for teacher in teachers:
            teacher_choices.append(
                (
                    teacher.id,
                    teacher.new_user.first_name + " " + teacher.new_user.last_name,
                )
            )
        super(ClassMoveForm, self).__init__(*args, **kwargs)
        self.fields["new_teacher"].choices = teacher_choices


class TeacherEditStudentForm(forms.Form):
    name = forms.CharField(
        label="Name", widget=forms.TextInput(attrs={"placeholder": "Name"})
    )

    def __init__(self, student, *args, **kwargs):
        self.student = student
        self.klass = student.class_field
        super(TeacherEditStudentForm, self).__init__(*args, **kwargs)

    def clean_name(self):
        name = stripStudentName(self.cleaned_data.get("name", ""))

        if name == "":
            raise forms.ValidationError(
                "'" + self.cleaned_data.get("name", "") + "' is not a valid name"
            )

        if re.match(re.compile("^[\w -]+$"), name) is None:
            raise forms.ValidationError(
                "Names may only contain letters, numbers, dashes, underscores, and spaces."
            )

        students = Student.objects.filter(
            class_field=self.klass, new_user__first_name__iexact=name
        )
        if students.exists() and students[0] != self.student:
            raise forms.ValidationError(
                "There is already a student called '" + name + "' in this class"
            )

        return name


class TeacherSetStudentPass(forms.Form):
    password = forms.CharField(
        label="New password",
        widget=forms.PasswordInput(attrs={"placeholder": "New password"}),
    )
    confirm_password = forms.CharField(
        label="Confirm new password",
        widget=forms.PasswordInput(attrs={"placeholder": "Confirm new password"}),
    )

    def clean_password(self):
        return form_clean_password(self, forms, "password")

    def clean(self):
        password = self.cleaned_data.get("password", None)
        confirm_password = self.cleaned_data.get("confirm_password", None)

        check_passwords(password, confirm_password)

        return self.cleaned_data


def validateStudentNames(klass, names):
    validationErrors = []

    if klass:
        # We want to report if a student already exists with that name.
        # But only report each name once if there are duplicates.
        students = Student.objects.filter(class_field=klass)
        clashes_found = []
        find_clashes(names, students, clashes_found, validationErrors)

    # Also report if a student appears twice in the list to be added.
    # But again only report each name once.
    lower_names = [name.lower() for name in names]
    find_duplicates(names, lower_names, validationErrors)

    find_illegal_characters(names, validationErrors)

    return validationErrors


def find_clashes(names, students, clashes_found, validationErrors):
    for name in names:
        if (
            students.filter(new_user__first_name__iexact=name).exists()
            and name not in clashes_found
        ):
            validationErrors.append(
                forms.ValidationError(
                    "There is already a student called '" + name + "' in this class"
                )
            )
            clashes_found.append(name)


def find_duplicates(names, lower_names, validationErrors):
    duplicates_found = []
    for duplicate in [name for name in names if lower_names.count(name.lower()) > 1]:
        if duplicate not in duplicates_found:
            validationErrors.append(
                forms.ValidationError(
                    "You cannot add more than one student called '" + duplicate + "'"
                )
            )
            duplicates_found.append(duplicate)


def find_illegal_characters(names, validationErrors):
    for name in names:
        if re.match(re.compile("^[\w -]+$"), name) is None:
            validationErrors.append(
                forms.ValidationError(
                    "Names may only contain letters, numbers, dashes, underscores, and spaces. You must rename '"
                    + name
                    + "'."
                )
            )


def check_passwords(password, confirm_password):
    if password is not None and password != confirm_password:
        raise forms.ValidationError(
            "The password and the confirmation password do not match"
        )


class TeacherMoveStudentsDestinationForm(forms.Form):
    new_class = forms.ChoiceField(
        label="Choose a new class from the drop down menu for the selected students.",
        widget=forms.Select(attrs={"class": "wide"}),
    )

    def __init__(self, classes, *args, **kwargs):
        self.classes = classes
        class_choices = []
        for klass in classes:
            class_choices.append(
                (
                    klass.id,
                    klass.name
                    + " ("
                    + klass.access_code
                    + "), "
                    + klass.teacher.new_user.first_name
                    + " "
                    + klass.teacher.new_user.last_name,
                )
            )
        super(TeacherMoveStudentsDestinationForm, self).__init__(*args, **kwargs)
        self.fields["new_class"].choices = class_choices


class TeacherMoveStudentDisambiguationForm(forms.Form):
    orig_name = forms.CharField(
        label="Original Name",
        widget=forms.TextInput(
            attrs={
                "readonly": "readonly",
                "placeholder": "Original Name",
                "type": "hidden",
            }
        ),
    )
    name = forms.CharField(
        label="Name",
        widget=forms.TextInput(attrs={"placeholder": "Name", "style": "margin : 0px"}),
    )

    def clean_name(self):
        name = stripStudentName(self.cleaned_data.get("name", ""))
        if name == "":
            raise forms.ValidationError(
                "'" + self.cleaned_data.get("name", "") + "' is not a valid name"
            )
        return name


class BaseTeacherMoveStudentsDisambiguationFormSet(forms.BaseFormSet):
    def __init__(self, destination, *args, **kwargs):
        self.destination = destination
        super(BaseTeacherMoveStudentsDisambiguationFormSet, self).__init__(
            *args, **kwargs
        )

    def clean(self):
        if any(self.errors):
            return

        names = [form.cleaned_data["name"] for form in self.forms]

        validationErrors = validateStudentNames(self.destination, names)

        if len(validationErrors) > 0:
            raise forms.ValidationError(validationErrors)

        self.strippedNames = names


class TeacherDismissStudentsForm(forms.Form):
    orig_name = forms.CharField(
        label="Original Name",
        widget=forms.TextInput(
            attrs={
                "readonly": "readonly",
                "placeholder": "Original Name",
                "style": "background-color: lightgray; margin: 0; border: 0",
            }
        ),
    )
    name = forms.CharField(
        label="New Name",
        widget=forms.TextInput(
            attrs={"placeholder": "New Name", "style": "margin : 0px"}
        ),
    )
    email = forms.EmailField(
        label="Email",
        widget=forms.EmailInput(
            attrs={"placeholder": "Email Address", "style": "margin : 0px"}
        ),
    )
    confirm_email = forms.EmailField(
        label="Confirm Email",
        widget=forms.EmailInput(
            attrs={"placeholder": "Confirm Email Address", "style": "margin : 0px"}
        ),
    )

    def clean_name(self):
        name = stripStudentName(self.cleaned_data.get("name", ""))

        if name == "":
            raise forms.ValidationError(
                "'" + self.cleaned_data.get("name", "") + "' is not a valid name"
            )

        if User.objects.filter(username=name).exists():
            raise forms.ValidationError("That username is already in use")

        return name

    def clean(self):
        email = self.cleaned_data.get("email", None)
        confirm_email = self.cleaned_data.get("confirm_email", None)

        if (email or confirm_email) and email != confirm_email:
            raise forms.ValidationError("Your new emails do not match")

        return self.cleaned_data


class BaseTeacherDismissStudentsFormSet(forms.BaseFormSet):
    def clean(self):
        if any(self.errors):
            return

        names = [form.cleaned_data["name"] for form in self.forms]

        validationErrors = validateStudentNames(None, names)

        if len(validationErrors) > 0:
            raise forms.ValidationError(validationErrors)


class StudentCreationForm(forms.Form):
    names = forms.CharField(
        label="names",
        widget=forms.Textarea(
            attrs={
                "placeholder": "You may type students names or copy and paste them from a spreadsheet into this text box."
            }
        ),
    )

    def __init__(self, klass, *args, **kwargs):
        self.klass = klass
        super(StudentCreationForm, self).__init__(*args, **kwargs)

    def clean(self):
        names = re.split(";|,|\n", self.cleaned_data.get("names", ""))
        names = list(map(stripStudentName, names))
        names = [name for name in names if name != ""]

        validationErrors = validateStudentNames(self.klass, names)

        if len(validationErrors) > 0:
            raise forms.ValidationError(validationErrors)

        self.strippedNames = names

        return self.cleaned_data


class TeacherAddExternalStudentForm(forms.Form):
    name = forms.CharField(
        label="Student name", widget=forms.TextInput(attrs={"placeholder": "Name"})
    )

    def __init__(self, klass, *args, **kwargs):
        self.klass = klass
        super(TeacherAddExternalStudentForm, self).__init__(*args, **kwargs)

    def clean_name(self):
        name = stripStudentName(self.cleaned_data.get("name", ""))

        if name == "":
            raise forms.ValidationError(
                "'" + self.cleaned_data.get("name", "") + "' is not a valid name"
            )

        if Student.objects.filter(
            class_field=self.klass, new_user__first_name__iexact=name
        ).exists():
            raise forms.ValidationError(
                "There is already a student called '" + name + "' in this class"
            )

        return name



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
"""Portal autoconfig"""
import os

from django_autoconfig.autoconfig import OrderingRelationship

DEFAULT_SETTINGS = {
    "AUTOCONFIG_INDEX_VIEW": "home",
    "LANGUAGE_CODE": "en-gb",
    "SITE_ID": 1,
    "MEDIA_ROOT": os.path.join(
        os.path.join(os.path.dirname(__file__), "static"), "email_media/"
    ),
    "MEDIA_URL": "/media/",
}

SETTINGS = {
    "AUTOCONFIG_DISABLED_APPS": [
        "django_otp",
        "django_otp.plugins.otp_static",
        "django_otp.plugins.otp_totp",
    ],
    "WAGTAIL_SITE_NAME": "Code for Life",
    "PIPELINE": {
        "COMPILERS": ("portal.pipeline_compilers.LibSassCompiler",),
        "STYLESHEETS": {
            "css": {
                "source_filenames": (
                    "portal/sass/bootstrap.scss",
                    "portal/sass/colorbox.scss",
                    "portal/sass/styles.scss",
                ),
                "output_filename": "portal.css",
            },
            "base": {
                "source_filenames": ("portal/sass/old_styles.scss",),
                "output_filename": "base.css",
            },
        },
        "CSS_COMPRESSOR": None,
        "SASS_ARGUMENTS": "--quiet",
    },
    "STATICFILES_FINDERS": ["pipeline.finders.PipelineFinder"],
    "STATICFILES_STORAGE": "pipeline.storage.PipelineStorage",
    "INSTALLED_APPS": [
        "aimmo",
        "game",
        "pipeline",
        "portal",
        "common",
        "django.contrib.admin",
        "django.contrib.admindocs",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.sites",
        "django.contrib.staticfiles",
        "rest_framework",
        "django_otp",
        "django_otp.plugins.otp_static",
        "django_otp.plugins.otp_totp",
        "sekizai",  # for javascript and css management
        "treebeard",
        "two_factor",
        "hijack",
        "compat",
        "hijack_admin",
        "wagtail.contrib.forms",
        "wagtail.contrib.redirects",
        "wagtail.embeds",
        "wagtail.sites",
        "wagtail.users",
        "wagtail.snippets",
        "wagtail.documents",
        "wagtail.images",
        "wagtail.search",
        "wagtail.admin",
        "wagtail.core",
        "modelcluster",
        "taggit",
    ],
    "LANGUAGES": [("en-gb", "English")],
    "MESSAGE_STORAGE": "django.contrib.messages.storage.session.SessionStorage",
    "MIDDLEWARE": [
        "django.contrib.sessions.middleware.SessionMiddleware",
        "django.middleware.locale.LocaleMiddleware",
        "django.middleware.common.CommonMiddleware",
        "django.middleware.csrf.CsrfViewMiddleware",
        "django.middleware.security.SecurityMiddleware",
        "django.contrib.auth.middleware.AuthenticationMiddleware",
        "django.contrib.auth.middleware.SessionAuthenticationMiddleware",
        "django.contrib.messages.middleware.MessageMiddleware",
        "django.middleware.clickjacking.XFrameOptionsMiddleware",
        "deploy.middleware.exceptionlogging.ExceptionLoggingMiddleware",
        "django_otp.middleware.OTPMiddleware",
        "wagtail.core.middleware.SiteMiddleware",
        "wagtail.contrib.redirects.middleware.RedirectMiddleware",
    ],
    "TEMPLATES": [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": [
                    "django.contrib.auth.context_processors.auth",
                    "django.template.context_processors.request",
                    "django.contrib.messages.context_processors.messages",
                    "sekizai.context_processors.sekizai",
                    "portal.context_processors.process_newsletter_form",
                    "portal.context_processors.module_name",
                ]
            },
        }
    ],
    "CODEFORLIFE_WEBSITE": "www.codeforlife.education",
    "CLOUD_STORAGE_PREFIX": "//storage.googleapis.com/codeforlife-assets/",
    "LOGGING": {
        "version": 1,
        "disable_existing_loggers": False,
        "handlers": {"console": {"level": "DEBUG", "class": "logging.StreamHandler"}},
        "loggers": {"two_factor": {"handlers": ["console"], "level": "INFO"}},
    },
    "RAPID_ROUTER_EARLY_ACCESS_FUNCTION_NAME": "portal.beta.has_beta_access",
    "SECURE_CONTENT_TYPE_NOSNIFF": True,
    "HIJACK_LOGIN_REDIRECT_URL": "/",
    "HIJACK_LOGOUT_REDIRECT_URL": "/administration/",
    "HIJACK_USE_BOOTSTRAP": True,
    "HIJACK_ALLOW_GET_REQUESTS": True,
}

RELATIONSHIPS = [
    OrderingRelationship(
        "MIDDLEWARE",
        "django_otp.middleware.OTPMiddleware",
        after=["django.contrib.auth.middleware.AuthenticationMiddleware"],
        add_missing=False,
    )
]

try:
    import django_pandasso

    SETTINGS["INSTALLED_APPS"].append("django_pandasso")
    SETTINGS["INSTALLED_APPS"].append("social.apps.django_app.default")
except ImportError:
    pass



# coding=utf-8
TEACH_BANNER = {
    "title": "Educate",
    "subtitle": "Get ready to teach the next generation of computer scientists",
    "text": "",
    "image_class": "banner--picture--educate",
}

TEACH_BENEFITS = {
    "first": {
        "image": "portal/img/icon_laptop.png",
        "title": "",
        "text": "<strong>Rapid Router is aligned to the UK Computing Curriculum</strong>, so you can gain the knowledge and confidence you need.",
        "button": {"text": "", "link": ""},
    },
    "second": {
        "image": "portal/img/icon_steps.png",
        "title": "",
        "text": "The step-by-step nature of Rapid Router makes it <strong>simple for you and your students</strong> to gain exerience quickly.",
        "button": {"text": "", "link": ""},
    },
    "third": {
        "image": "portal/img/icon_graph.png",
        "title": "",
        "text": "Easy to use teacher dashboard includes scores to <strong>track student progress</strong> as well as <strong>comprehensive teaching resources</strong>.",
        "button": {"text": "", "link": ""},
    },
}



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

from django.core.urlresolvers import reverse
from hamcrest import *
from hamcrest.core.base_matcher import BaseMatcher
from rest_framework import status
from rest_framework.test import APIClient, APITestCase

from common.tests.utils.user import create_user_directly, get_superuser


class APITests(APITestCase):
    def test_valid_date_registered(self):
        url = reverse(
            "registered-users", kwargs={"year": "2016", "month": "04", "day": "01"}
        )
        superuser = get_superuser()
        self.client.force_authenticate(user=superuser)
        response = self.client.get(url)
        assert_that(response, has_status_code(status.HTTP_200_OK))
        assert_that(isinstance(response.data, int))

    def test_invalid_date_registered(self):
        url = reverse(
            "registered-users", kwargs={"year": "2016", "month": "05", "day": "35"}
        )
        superuser = get_superuser()
        self.client.force_authenticate(user=superuser)
        response = self.client.get(url)
        assert_that(response, has_status_code(status.HTTP_404_NOT_FOUND))

    def test_valid_date_lastconnectedsince(self):
        url = reverse(
            "last-connected-since", kwargs={"year": "2016", "month": "04", "day": "01"}
        )
        superuser = get_superuser()
        self.client.force_authenticate(user=superuser)
        response = self.client.get(url)
        assert_that(response, has_status_code(status.HTTP_200_OK))
        assert_that(isinstance(response.data, int))

    def test_invalid_date_lastconnectedsince(self):
        url = reverse(
            "last-connected-since", kwargs={"year": "2016", "month": "05", "day": "35"}
        )
        superuser = get_superuser()
        self.client.force_authenticate(user=superuser)
        response = self.client.get(url)
        assert_that(response, has_status_code(status.HTTP_404_NOT_FOUND))

    def test_valid_country_userspercountry(self):
        url = reverse("number_users_per_country", kwargs={"country": "GB"})
        superuser = get_superuser()
        self.client.force_authenticate(user=superuser)
        response = self.client.get(url)
        assert_that(response, has_status_code(status.HTTP_200_OK))
        assert_that(isinstance(response.data, int))

    def test_get_inactive_users_if_admin(self):
        client = APIClient()
        superuser = get_superuser()
        create_user_directly(active=False)
        create_user_directly(active=True)
        client.force_authenticate(user=superuser)
        url = reverse("inactive_users")
        response = client.get(url)
        self.assertEqual(len(response.data), 1)

    def test_get_inactive_users_if_appengine(self):
        client = APIClient()
        create_user_directly(active=False)
        create_user_directly(active=True)
        url = reverse("inactive_users")
        client.credentials(HTTP_X_APPENGINE_CRON=True)
        response = client.get(url)
        self.assertEqual(len(response.data), 1)

    def test_get_inactive_users_if_unauthorised(self):
        client = APIClient()
        create_user_directly(active=False)
        create_user_directly(active=True)
        url = reverse("inactive_users")
        response = client.get(url)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_delete_inactive_users_if_appengine(self):
        client = APIClient()
        create_user_directly(active=False)
        create_user_directly(active=False)
        url = reverse("inactive_users")
        client.credentials(HTTP_X_APPENGINE_CRON=True)
        response = client.delete(url)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        response = client.get(url)
        self.assertEqual(len(response.data), 0)


def has_status_code(status_code):
    return HasStatusCode(status_code)


class HasStatusCode(BaseMatcher):
    def __init__(self, status_code):
        self.status_code = status_code

    def _matches(self, response):
        return response.status_code == self.status_code

    def describe_to(self, description):
        description.append_text("has status code ").append_text(self.status_code)

    def describe_mismatch(self, response, mismatch_description):
        mismatch_description.append_text("had status code ").append_text(
            response.status_code
        )



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

from portal.tests.pageObjects.portal.email_verification_needed_page import (
    EmailVerificationNeededPage,
)
from portal.tests.pageObjects.portal.play.dashboard_page import PlayDashboardPage
from .play_base_page import PlayBasePage


class PlayAccountPage(PlayBasePage):
    def __init__(self, browser):
        super(PlayAccountPage, self).__init__(browser)

        assert self.on_correct_page("play_account_page")

    def check_account_details(self, details):
        correct = True

        for field, value in list(details.items()):
            correct &= (
                self.browser.find_element_by_id("id_" + field).get_attribute("value")
                == value
            )

        return correct

    def _change_details(self, details):
        for field, value in list(details.items()):
            self.browser.find_element_by_id("id_" + field).clear()
            self.browser.find_element_by_id("id_" + field).send_keys(value)
        self.browser.find_element_by_id("update_button").click()

    def submit_empty_form(self):
        self.browser.find_element_by_id("update_button").click()
        return self

    def update_password_failure(self, new_password, confirm_new_password, old_password):
        self._update_password(new_password, confirm_new_password, old_password)
        return self

    def update_password_success(self, new_password, confirm_new_password, old_password):
        self._update_password(new_password, confirm_new_password, old_password)
        return PlayDashboardPage(self.browser)

    def update_name_failure(self, new_name, password):
        self._update_name(new_name, password)
        return self

    def update_name_success(self, new_name, password):
        self._update_name(new_name, password)
        return PlayDashboardPage(self.browser)

    def change_email(self, new_email, password):
        self._change_details({"email": new_email, "current_password": password})
        return EmailVerificationNeededPage(self.browser)

    def _update_password(self, new_password, confirm_new_password, old_password):
        self.browser.find_element_by_id("id_password").send_keys(new_password)
        self.browser.find_element_by_id("id_confirm_password").send_keys(
            confirm_new_password
        )
        self.browser.find_element_by_id("id_current_password").send_keys(old_password)
        self.browser.find_element_by_id("update_button").click()

    def _update_name(self, new_name, password):
        self.browser.find_element_by_id("id_name").clear()
        self.browser.find_element_by_id("id_name").send_keys(new_name)
        self.browser.find_element_by_id("id_current_password").send_keys(password)
        self.browser.find_element_by_id("update_button").click()



# -*- coding: utf-8 -*-

from builtins import str
from builtins import range
from django.contrib.auth.decorators import login_required, user_passes_test
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import render

from common.permissions import logged_in_as_teacher
from portal.strings.materials import MATERIALS_BANNER
from portal.views.teacher.pdfs import PDF_DATA


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def materials(request):

    session_names = [
        "KS1_session_",
        "KS2_session_",
        "Lower_KS3_session_",
        "Intermediate_KS3_session_",
        "Upper_KS3_session_",
    ]
    resource_sheets_names = [
        "KS1_S",
        "KS2_S",
        "Lower_KS3_S",
        "Intermediate_KS3_S",
        "Upper_KS3_S",
    ]
    ks1_sessions = []
    ks2_sessions = []
    lower_ks3_sessions = []
    intermediate_ks3_sessions = []
    upper_ks3_sessions = []
    session_lists = [
        ks1_sessions,
        ks2_sessions,
        lower_ks3_sessions,
        intermediate_ks3_sessions,
        upper_ks3_sessions,
    ]
    ks1_sheets = []
    ks2_sheets = []
    lower_ks3_sheets = []
    intermediate_ks3_sheets = []
    upper_ks3_sheets = []
    resource_sheets_lists = [
        ks1_sheets,
        ks2_sheets,
        lower_ks3_sheets,
        intermediate_ks3_sheets,
        upper_ks3_sheets,
    ]
    ks1_sheets_table = {}
    ks2_sheets_table = {}
    lower_ks3_sheets_table = {}
    intermediate_ks3_sheets_table = {}
    upper_ks3_sheets_table = {}
    resource_sheets_tables = [
        ks1_sheets_table,
        ks2_sheets_table,
        lower_ks3_sheets_table,
        intermediate_ks3_sheets_table,
        upper_ks3_sheets_table,
    ]

    for ks_index, session_name in enumerate(session_names):
        get_session_pdfs(session_name, session_lists[ks_index])
        get_resource_sheets_pdfs(
            session_lists[ks_index],
            resource_sheets_names[ks_index],
            resource_sheets_lists[ks_index],
            resource_sheets_tables[ks_index],
        )
        resource_sheets_tables[ks_index]["content"] = resource_sheets_lists[ks_index]

    return render(
        request,
        "portal/teach/materials.html",
        {
            "ks1_sessions": ks1_sessions,
            "ks1_sheets": ks1_sheets_table,
            "ks2_sessions": ks2_sessions,
            "ks2_sheets": ks2_sheets_table,
            "lower_ks3_sessions": lower_ks3_sessions,
            "lower_ks3_sheets": lower_ks3_sheets_table,
            "intermediate_ks3_sessions": intermediate_ks3_sessions,
            "intermediate_ks3_sheets": intermediate_ks3_sheets_table,
            "upper_ks3_sessions": upper_ks3_sessions,
            "upper_ks3_sheets": upper_ks3_sheets_table,
            "BANNER": MATERIALS_BANNER,
        },
    )


def get_session_pdfs(session_name, session_list):
    session_pdf_exists = True
    session_number = update_session_number_based_on_key_stage(session_name)

    while session_pdf_exists:
        pdf_name = session_name + str(session_number)

        try:
            pdf = PDF_DATA[pdf_name]
            pdf["session_number"] = session_number
            pdf["url_name"] = pdf_name
            session_list.append(pdf)
            session_number += 1
        except KeyError:
            session_pdf_exists = False


def get_resource_sheets_pdfs(
    session_list, resource_sheets_name, resource_sheets_list, resource_sheets_table=None
):
    """
    This function gathers all the resource sheets for each session in the session list. It finds the correct PDFs based
    on the session number and the prefix of the resource sheets name. In addition, if a resource sheets table is
    defined, it will add the starting session index to the dictionary.
    :param session_list: The list of sessions for which this function will gather resource sheets.
    :param resource_sheets_name: The prefix of the name of the required resource sheets' names.
    :param resource_sheets_list: The resource sheets list which is updated to contain the resource sheets.
    :param resource_sheets_table: Optional argument. If defined, it means the data is supposed to be shown in a table
    and requires a starting session index to be able to iterate properly.
    """
    starting_session_index = update_session_number_based_on_key_stage(
        resource_sheets_name
    )
    if resource_sheets_table is not None:
        resource_sheets_table["starting_session_index"] = starting_session_index

    for session_index in range(
        starting_session_index, len(session_list) + starting_session_index
    ):
        resource_pdf_exists = True
        resource_number = 1
        pdfs = []

        while resource_pdf_exists:
            pdf_name = (
                resource_sheets_name + str(session_index) + "_" + str(resource_number)
            )

            try:
                pdf = PDF_DATA[pdf_name]
                pdf["session_number"] = session_index
                pdf["url_name"] = pdf_name
                pdfs.append(pdf)
                resource_number += 1

            except KeyError:
                resource_pdf_exists = False

        resource_sheets_list.append(pdfs)


def update_session_number_based_on_key_stage(key_stage_name):
    """
    Defines the correct starting session index based on which key stage is specified.
    :param key_stage_name: Prefix of the specified key stage's session or resource sheets name.
    :return: Returns the appropriate starting session index.
    """
    DEFAULT_KS_STARTING_SESSION_INDEX = 1
    INTERMEDIATE_KS3_STARTING_SESSION_INDEX = 6
    UPPER_KS3_STARTING_SESSION_INDEX = 11

    if (
        key_stage_name == "Intermediate_KS3_session_"
        or key_stage_name == "Intermediate_KS3_S"
    ):
        session_number = INTERMEDIATE_KS3_STARTING_SESSION_INDEX
    elif key_stage_name == "Upper_KS3_session_" or key_stage_name == "Upper_KS3_S":
        session_number = UPPER_KS3_STARTING_SESSION_INDEX
    else:
        session_number = DEFAULT_KS_STARTING_SESSION_INDEX

    return session_number



from .base_page import BasePage
from .play import dashboard_page
from .student_password_reset_form_page import StudentPasswordResetFormPage
from .teach import dashboard_page as teach_dashboard_page
from .teach import onboarding_classes_page
from .teach import onboarding_students_page
from .teacher_password_reset_form_page import TeacherPasswordResetFormPage
import time


class IndependentStudentLoginPage(BasePage):
    def __init__(self, browser):
        super(IndependentStudentLoginPage, self).__init__(browser)

        assert self.on_correct_page("independent_login_page")

    def independent_student_login(self, username, password):
        self._independent_student_login(username, password)

        return dashboard_page.PlayDashboardPage(self.browser)

    def independent_student_login_failure(self, username, password):
        self._independent_student_login(username, password)
        return self

    def _independent_student_login(self, username, password):
        self.browser.find_element_by_id("id_username").send_keys(username)
        self.browser.find_element_by_id("id_password").send_keys(password)
        self.browser.find_element_by_name("independent_student_login").click()

    def go_to_indep_forgotten_password_page(self):
        self.browser.find_element_by_id("student_forgotten_password_button").click()
        return StudentPasswordResetFormPage(self.browser)

    def has_login_failed(self, form_id, error):
        time.sleep(1.5)
        errors = (
            self.browser.find_element_by_id(form_id)
            .find_element_by_class_name("errorlist")
            .text
        )
        return error in errors



# coding=utf-8
RAPID_ROUTER_HEADLINE = {
    "title": "Anyone can code, you can too!",
    "description": "Whether you’re a teacher or a student, Rapid Router guides you, "
    "and makes learning to code easy and great fun. Using Blockly you can "
    "advance through the levels to become an Ocado delivery hero.",
}

RAPID_ROUTER_BENEFITS = {
    "first": {
        "image": "portal/img/teaching_tools.jpg",
        "title": "Free teaching tools",
        "text": "Free teaching resources, lesson plans and much more. You can even "
        "track your student’s progress with our class tracker.",
        "button": {"text": "Learn more", "link": "teach"},
    },
    "second": {
        "image": "portal/img/a_fun_game_for_students.jpg",
        "title": "A fun game for students",
        "text": "You can learn to become a coding superhero, amaze your friends with "
        "your high scores.",
        "button": {"text": "Play now", "link": "levels"},
    },
    "third": {
        "image": "portal/img/about_blockly.png",
        "title": "About Blockly",
        "text": "Built on ‘Blockly’, an easy-to-use visual programming language "
        "that’s similar to Scratch, so you don’t need to worry about syntax.",
        "button": {"text": "", "link": ""},
    },
}



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.

from common.models import Student
from django.contrib import messages as messages
from django.contrib.auth import logout
from django.contrib.auth.decorators import login_required, user_passes_test
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.urlresolvers import reverse_lazy
from django.http import HttpResponseRedirect
from django.views.generic.edit import FormView

from portal.forms.play import StudentEditAccountForm, IndependentStudentEditAccountForm
from common.helpers.emails import update_email
from portal.helpers.password import check_update_password
from common.permissions import logged_in_as_student


def _get_form(self, form_class):
    """
    Generic function which gets the appropriate edit account details form for either
    a school student or an independent student.
    :param self: The view class the form is used in.
    :param form_class: The form class which corresponds to the type of student.
    :return: The initialised form which can then be used in the FormView.
    """
    user = self.request.user
    if form_class is None:
        form_class = self.get_form_class()
    return form_class(user, **self.get_form_kwargs())


def _process_form(self, process_function, form, view):
    """
    Generic function which processes the appropriate edit account details form for
    either a school student or an independent student.
    :param self: The view class the form is used in.
    :param process_function: The core function that is needed to process the form.
    :param form: The form which needs to be processed.
    :param view: The view object.
    :return: The view once the form is valid and has been submitted.
    """
    student = self.request.user.new_student
    process_function(form, student, self.request)
    return super(view, self).form_valid(form)


class SchoolStudentEditAccountView(LoginRequiredMixin, FormView):
    """
    A FormView for editing a school student's account details. This forms enables a
    school student to change their password.
    """

    login_url = reverse_lazy("student_login")
    form_class = StudentEditAccountForm
    template_name = "../templates/portal/play/student_edit_account.html"
    success_url = reverse_lazy("student_details")
    model = Student

    def form_valid(self, form):
        return _process_form(
            self,
            self.process_student_edit_account_form,
            form,
            SchoolStudentEditAccountView,
        )

    def process_student_edit_account_form(self, form, student, request):
        data = form.cleaned_data
        # check not default value for CharField
        check_update_password(form, student.new_user, request, data)

        messages.success(
            request, "Your account details have been changed successfully."
        )

    def get_form(self, form_class=None):
        return _get_form(self, form_class)


class IndependentStudentEditAccountView(LoginRequiredMixin, FormView):
    """
    A FormView for editing an independent student's account details. This forms enables
    an independent student to change their name, their email and / or their password.
    """

    login_url = reverse_lazy("independent_student_login")
    form_class = IndependentStudentEditAccountForm
    template_name = "../templates/portal/play/student_edit_account.html"
    model = Student
    initial = {"name": "Could not find name"}
    changing_email = False

    def get_form_kwargs(self):
        kwargs = super(IndependentStudentEditAccountView, self).get_form_kwargs()
        kwargs["initial"]["name"] = "{}{}".format(
            self.request.user.first_name, self.request.user.last_name
        )
        return kwargs

    def get_form(self, form_class=None):
        return _get_form(self, form_class)

    def get_success_url(self):
        if self.changing_email:
            return reverse_lazy("email_verification")
        else:
            return reverse_lazy("student_details")

    def form_valid(self, form):
        return _process_form(
            self,
            self.process_independent_student_edit_account_form,
            form,
            IndependentStudentEditAccountView,
        )

    def process_independent_student_edit_account_form(self, form, student, request):
        data = form.cleaned_data

        # check not default value for CharField
        check_update_password(form, student.new_user, request, data)

        # allow individual students to update more
        self.changing_email, new_email = update_email(student.new_user, request, data)

        self.update_name(student, data)

        messages.success(
            request, "Your account details have been changed successfully."
        )

        if self.changing_email:
            logout(request)

    def update_name(self, student, data):
        student.new_user.first_name = data["name"]
        # save all tables
        student.save()
        student.new_user.save()


@login_required(login_url=reverse_lazy("home"))
@user_passes_test(logged_in_as_student, login_url=reverse_lazy("home"))
def student_edit_account(request):
    student = request.user.new_student
    if student.is_independent():
        return HttpResponseRedirect(reverse_lazy("independent_edit_account"))
    else:
        return HttpResponseRedirect(reverse_lazy("school_student_edit_account"))



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2018, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

from selenium.webdriver.common.action_chains import ActionChains

from . import edit_student_password_page
from .teach_base_page import TeachBasePage


class EditStudentPage(TeachBasePage):
    def __init__(self, browser):
        super(EditStudentPage, self).__init__(browser)

        assert self.on_correct_page("edit_student_page")

    def type_student_name(self, name):
        self.browser.find_element_by_id("id_name").clear()
        self.browser.find_element_by_id("id_name").send_keys(name)
        return self

    def type_student_password(self, password):
        self.browser.find_element_by_id("id_password").send_keys(password)
        self.browser.find_element_by_id("id_confirm_password").send_keys(password)
        return self

    def click_update_button(self):
        self.browser.find_element_by_id("update_name_button").click()
        return self

    def click_set_password_form_button(self):
        self.browser.find_element_by_id("request-password-setter").click()
        return self

    def click_set_password_button(self):
        self.browser.find_element_by_id("set_new_password_button").click()
        return edit_student_password_page.EditStudentPasswordPage(self.browser)

    def click_generate_password_button(self):
        actions = ActionChains(self.browser)
        generate_password_button = self.browser.find_element_by_id(
            "generate_password_button"
        )
        actions.move_to_element(generate_password_button).click()
        actions.perform()
        return edit_student_password_page.EditStudentPasswordPage(self.browser)

    def is_student_name(self, name):
        return name in self.browser.find_element_by_id("student_details").text
